//------------------------------------------------------------------------------------------------
modded class SCR_PlayerController : PlayerController
{
	static const float FOCUS_ACTIVATION = 0.05;
	static const float FOCUS_DEACTIVATION = 0.01;
	static const float FOCUS_TIMEOUT = 0.5;
	static const float FOCUS_STEP = 0.005;
	static float s_fFocusAnalogue;

	//------------------------------------------------------------------------------------------------
	/*! Focus input degree for analogue input
	\param adsProgress ADS focus percentage
	\param dt Delta time
	\return focus Amount of focus between 0 and 1
	*/
	override float GetFocusValue(float adsProgress = 0, float dt = -1)
	{
		if (!m_CharacterController)
			return 0;

		float focus;

		// Autofocus
		if (adsProgress > 0)
			focus = s_fADSFocus * Math.Min(adsProgress, 1);

		InputManager inputManager = GetGame().GetInputManager();

		// Cancel toggled focus when focus is held
		bool inputDigital = inputManager.GetActionTriggered("Focus");
		if (inputDigital && m_bFocusToggle)
			m_bFocusToggle = false;

		// Conditions must be consistent with ActionFocusToggle and ActionFocusToggleUnarmed
		ChimeraCharacter character = m_CharacterController.GetCharacter();
		if (m_bFocusToggle && character)
		{
			if (character.IsInVehicle())
			{
				// Cancel toggle focus when in vehicle and aiming through gadget (binocular, compass)
				if (m_bGadgetFocus)
					m_bFocusToggle = false;

				// Cancel toggle focus when in vehicle and not in forced freelook
				if (!m_CharacterController.IsFreeLookEnforced())
					m_bFocusToggle = false;
			}
			else
			{
				// Cancel toggle focus when not in vehicle and holding item in hands
				if (m_CharacterController.GetCurrentItemInHands())
					m_bFocusToggle = false;

				// Cancel toggle focus when not in vehicle and holding gadget
				if (m_CharacterController.IsGadgetInHands())
					m_bFocusToggle = false;
			}
		}

		// Ground vehicles have different focus action to prevent conflict with brakes
		float inputAnalogue;
		if (!inputManager.IsContextActive("CarContext") && !inputManager.IsContextActive("HelicopterContext"))
		{
			// Square root input to focus mapping results in linear change of picture area
			float focusAnalogue = Math.Sqrt(inputManager.GetActionValue("FocusAnalog"));

			// Tolerance to prevent jittering
			if (focusAnalogue < FOCUS_STEP)
				s_fFocusAnalogue = 0;
			else if (!float.AlmostEqual(s_fFocusAnalogue, focusAnalogue, FOCUS_STEP))
				s_fFocusAnalogue = focusAnalogue;
			
			inputAnalogue = s_fFocusAnalogue;
		}

		// Prevent focus warping back while toggling ADS on controller
		// analogue: track timeout as we have no input filter that has thresholds or delays and returns axis value yet
		if (inputAnalogue <= FOCUS_DEACTIVATION)
			s_fFocusTimeout = FOCUS_TIMEOUT; // Below deactivation threshold
		else if (inputAnalogue < FOCUS_ACTIVATION && s_fFocusTimeout > 0)
			s_fFocusTimeout = FOCUS_TIMEOUT;  // Below activation threshold and not active
		else if (s_fFocusTimeout > dt)
			s_fFocusTimeout -= dt; // Not yet active, decrementing
		else
			s_fFocusTimeout = 0; // Activated

		// Cancel toggle focus with analogue input
		if (m_bFocusToggle && s_fFocusTimeout == 0)
			m_bFocusToggle = false;

		// Combine all valid input sources
		float input;
		if (m_bFocusToggle || inputDigital)
			input = 1;
		else if (s_fFocusTimeout == 0)
			input = inputAnalogue;

		if (input > 0)
			focus = Math.Max(focus, input);

		// Ensure return value always within 0-1
		focus = Math.Clamp(focus, 0, 1);
		return focus;
	}
}
